@@ -1,5 +1,5 @@
 (*
- * Copyright (c) 1997,1998 Massachusetts Institute of Technology
+ * Copyright (c) 1997-1999 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *
  *)
 
-(* $Id: exprdag.ml,v 1.35 1998/09/25 19:14:16 athena Exp $ *)
-let cvsid = "$Id: exprdag.ml,v 1.35 1998/09/25 19:14:16 athena Exp $"
+(* $Id: exprdag.ml,v 1.41 1999/05/26 15:44:14 fftw Exp $ *)
+let cvsid = "$Id: exprdag.ml,v 1.41 1999/05/26 15:44:14 fftw Exp $"
 
 open Util
 
@@ -52,6 +52,54 @@ end
 
 open Hash
 
+module LittleSimplifier = struct 
+  (* 
+   * The LittleSimplifier module implements a subset of the simplifications
+   * of the AlgSimp module.  These simplifications can be executed
+   * quickly here, while they would take a long time using the heavy
+   * machinery of AlgSimp.  
+   * 
+   * For example, 0 * x is simplified to 0 tout court by the LittleSimplifier.
+   * On the other hand, AlgSimp would first simplify x, generating lots
+   * of common subexpressions, storing them in a table etc, just to
+   * discard all the work later.  Similarly, the LittleSimplifier
+   * reduces the constant FFT in Rader's algorithm to a constant sequence.
+   *)
+  let rec makeNum = function
+    | n -> Num n
+
+  and makeUminus = function
+    | Uminus a -> a 
+    | Num a -> makeNum (Number.negate a)
+    | a -> Uminus a
+
+  and makeTimes = function
+    | (Num a, Num b) -> makeNum (Number.mul a b)
+    | (Num a, Times (Num b, c)) -> makeTimes (makeNum (Number.mul a b), c)
+    | (Num a, b) when Number.is_zero a -> makeNum (Number.zero)
+    | (Num a, b) when Number.is_one a -> b
+    | (Num a, b) when Number.is_mone a -> makeUminus b
+    | (Num a, Uminus b) -> Times (makeUminus (Num a), b)
+    | (a, (Num b as b')) -> makeTimes (b', a)
+    | (a, b) -> Times (a, b)
+
+  and makePlus l = 
+    let rec reduceSum x = match x with
+	[] -> []
+      | [Num a] -> if Number.is_zero a then [] else x
+      | (Num a) :: (Num b) :: c -> 
+	  reduceSum ((makeNum (Number.add a b)) :: c)
+      | ((Num _) as a') :: b :: c -> b :: reduceSum (a' :: c)
+      | a :: s -> a :: reduceSum s
+    in match reduceSum l with
+      [] -> makeNum (Number.zero)
+    | [a] -> a 
+    | [a; b] when a == b -> makeTimes (Num Number.two, a)
+    | [Times (Num a, b); Times (Num c, d)] when b == d ->
+ 	makeTimes (makePlus [Num a; Num c], b)
+    | a -> Plus a
+end
+
 (*************************************************************
  *    Functional associative table
  *************************************************************)
@@ -189,6 +237,8 @@ end  = struct
 
   let memoizing_variables = make_memoizer hash_variable Variable.same
 
+  let memoizing_nodes = make_memoizer hash_node (==)
+
   let random_oracle =
     memoizing_variables
       (fun _ -> (float (Random.bits())) /. 1073741824.0)
@@ -196,13 +246,15 @@ end  = struct
 
   let sum_list l = List.fold_right (+.) l 0.0
 
-  let rec eval = function
-      Num x -> Number.to_float x
-    | Load v -> random_oracle v
-    | Store (v, x) -> random_oracle v
-    | Plus l -> sum_list (List.map eval l)
-    | Times (a, b) -> (eval a) *. (eval b)
-    | Uminus x -> -. (eval x)
+  let rec eval x =
+    memoizing_nodes (function
+      	Num x -> Number.to_float x
+      | Load v -> random_oracle v
+      | Store (v, x) -> random_oracle v
+      | Plus l -> sum_list (List.map eval l)
+      | Times (a, b) -> (eval a) *. (eval b)
+      | Uminus x -> -. (eval x) )
+      x
 
   let should_flip_sign node = 
     let v = eval node in
@@ -214,6 +266,7 @@ module Reverse = struct
   open StateMonad
   open MemoMonad
   open AssocTable
+  open LittleSimplifier
 
   let fetchDuals = fetchState
   let storeDuals = storeState
@@ -226,11 +279,11 @@ module Reverse = struct
     fetchDuals >>= fun table ->
       storeDuals (node_insert key value table)
 
-  let rec visit visited vtable = function
-      [] -> visited
+  let rec visit visited vtable parent_table = function
+      [] -> (visited, parent_table)
     | node :: rest ->
 	match AssocTable.lookup hash_node (==) node vtable with
-	  Some _ -> visit visited vtable rest
+	  Some _ -> visit visited vtable parent_table rest
 	| None ->
 	    let children = match node with
 	      Store (v, n) -> [n]
@@ -238,11 +291,25 @@ module Reverse = struct
 	    | Times (a, b) -> [a; b]
 	    | Uminus x -> [x]
 	    | _ -> []
-	    in visit (node :: visited) 
+	    in let rec loop t = function
+		[] -> t
+	      |	a :: rest ->
+		  (match AssocTable.lookup hash_node (==) a t with
+		    None -> 
+		      loop 
+			(AssocTable.insert hash_node a [node] t)
+			rest
+		  | Some c ->
+		      loop 
+			(AssocTable.insert hash_node a (node :: c) t)
+			rest)
+	    in visit 
+	      (node :: visited)
 	      (AssocTable.insert hash_node node () vtable)
+	      (loop parent_table children)
 	      (children @ rest)
 
-  let make_reverser all_nodes =
+  let make_reverser parent_table =
     let rec termM node candidate_parent = 
       match candidate_parent with
 	Store (_, n) when n == node -> 
@@ -251,15 +318,19 @@ module Reverse = struct
 	  dualM candidate_parent >>= fun x' -> returnM [x']
       | Times (a, b) when b == node -> 
 	  dualM candidate_parent >>= fun x' -> 
-	    returnM [Times (a, x')]
+	    returnM [makeTimes (a, x')]
       | Uminus n when n == node -> 
 	  dualM candidate_parent >>= fun x' -> 
-	    returnM [Uminus x']
+	    returnM [makeUminus x']
       | _ -> returnM []
     
     and dualExpressionM this_node = 
-      mapM (termM this_node) all_nodes >>= fun l ->
-	returnM (Plus (List.flatten l))
+      mapM (termM this_node) 
+	(match AssocTable.lookup hash_node (==) this_node parent_table with
+	  Some a -> a
+	| None -> failwith "bug in dualExpressionM"
+	) >>= fun l ->
+	returnM (makePlus (List.flatten l))
 
     and dualM this_node =
       memoizing lookupDualsM insertDualsM
@@ -281,15 +352,63 @@ module Reverse = struct
     | _ -> false
 
   let reverse (Dag dag) = 
-    let all_nodes = visit [] empty dag in
-    let reverserM = make_reverser all_nodes in
+    let (all_nodes, parent_table) = visit [] empty empty dag in
+    let reverserM = make_reverser parent_table in
     let mapReverserM = mapM reverserM in
     let duals = runM empty mapReverserM all_nodes in
     let roots = filter is_store duals
     in Dag roots
 end
 
- 
+(*************************************************************
+ * Various dag statistics
+ *************************************************************)
+module Stats : sig
+  val complexity : dag -> string
+end = struct
+  let rec visit visited vtable = function
+      [] -> visited
+    | node :: rest ->
+	match AssocTable.lookup hash_node (==) node vtable with
+	  Some _ -> visit visited vtable rest
+	| None ->
+	    let children = match node with
+	      Store (v, n) -> [n]
+	    | Plus l -> l
+	    | Times (a, b) -> [a; b]
+	    | Uminus x -> [x]
+	    | _ -> []
+	    in visit (node :: visited)
+	      (AssocTable.insert hash_node node () vtable)
+	      (children @ rest)
+
+  let complexity (Dag dag) = 
+    let rec loop (load, store, plus, times, uminus, num) = function 
+      	[] -> (load, store, plus, times, uminus, num)
+      | node :: rest ->
+	  loop
+	    (match node with
+	      Load _ -> (load + 1, store, plus, times, uminus, num)
+	    | Store _ -> (load, store + 1, plus, times, uminus, num)
+	    | Plus _ -> (load, store, plus + 1, times, uminus, num)
+	    | Times _ -> (load, store, plus, times + 1, uminus, num)
+	    | Uminus _ -> (load, store, plus, times, uminus + 1, num)
+	    | Num _ -> (load, store, plus, times, uminus, num + 1))
+	    rest
+    in let (l, s, p, t, u, n) = 
+      loop (0, 0, 0, 0, 0, 0) (visit [] AssocTable.empty dag)
+    in 
+    "This dag contains: " ^
+    (string_of_int l) ^ " loads, " ^
+    (string_of_int s) ^ " stores, " ^
+    (string_of_int p) ^ " plus, " ^
+    (string_of_int t) ^ " times, " ^
+    (string_of_int u) ^ " uminus, " ^
+    (string_of_int n) ^ " num, " ^
+    (string_of_int (l + s + p + t + u + n) ^ " total nodes.")
+end    
+  
+
 (*************************************************************
  * Algebraic simplifier/elimination of common subexpressions
  *************************************************************)
@@ -477,12 +596,13 @@ end = struct
 		stimesM (w', xpr) >>= fun t' ->
 		  returnM (t':: wo')
 
-  and mangleSumM x = 
-    reduce_sumM x 
-      >>= collectCoeffM 
-      >>= collectExprM 
+  and mangleSumM x = returnM x
       >>= reduce_sumM 
-      >>= eliminateObviousButterfliesM 
+      >>= collectExprM 
+      >>= collectCoeffM 
+      >>= reduce_sumM 
+      >>= eliminateButterflyishPatternsM
+      >>= reduce_sumM
 
   and reorder_uminus = function  (* push all Uminuses to the end *)
       [] -> []
@@ -498,38 +618,49 @@ end = struct
       Uminus _ -> true
     | _ -> false
 
-  and eliminateObviousButterfliesM = 
-    let moduloUminus = function
-	Uminus x -> x
-      |	x -> x
-    in let simplifiable a b c d =
-      let a' = moduloUminus a 
-      and b' = moduloUminus b 
-      and c' = moduloUminus c 
-      and d' = moduloUminus d 
-      in ((a' == c' && b' == d') or (b' == c' && a' == d'))
-
-    in function
-	([Plus [a; b]; Plus [c; d]]) when simplifiable a b c d ->
-	  mangleSumM [a; b; c; d]
-      |	([Uminus (Plus [a; b]); Plus [c; d]])
-	  when simplifiable a b c d ->
-	    suminusM a >>= fun a' ->
-	      suminusM b >>= fun b' ->
-		mangleSumM [a'; b'; c; d]
-      |	([Plus [c; d]; Uminus (Plus [a; b])])
-	  when simplifiable a b c d ->
-	    suminusM a >>= fun a' ->
-	      suminusM b >>= fun b' ->
-		mangleSumM [a'; b'; c; d]
-      |	([Uminus (Plus [a; b]); Uminus (Plus [c; d])])
-	  when simplifiable a b c d ->
-	    suminusM a >>= fun a' ->
-	      suminusM b >>= fun b' ->
-		suminusM c >>= fun c' ->
-		  suminusM d >>= fun d' ->
-		mangleSumM [a'; b'; c'; d']
-      | x -> returnM x
+  (*
+   * simplify patterns of the form
+   *
+   *  (c_1 * a + ...) +  (c_2 * a + ...)
+   *
+   * The pattern includes arbitrary coefficients and minus signs.
+   * A common case of this pattern is the butterfly
+   *   (a + b) + (a - b)
+   *   (a + b) - (a - b)
+   *)
+  and eliminateButterflyishPatternsM l =
+    let rec findTerms depth x = match x with
+      | Uminus x -> findTerms depth x
+      |	Times (Num a, b) -> findTerms (depth - 1) b
+      |	Plus l when depth > 0 ->
+	  x :: List.flatten (List.map (findTerms (depth - 1)) l)
+      |	x -> [x]
+    and duplicates = function
+	[] -> []
+      |	a :: b -> if List.memq a b then a :: duplicates b
+      else duplicates b
+    in let rec flattenPlusM d coef x =
+      if (List.memq x d) then
+ 	snumM coef >>= fun coef' ->
+	  stimesM (coef', x) >>= fun x' -> returnM [x']
+      else match x with
+      |	Times (Num a, b) ->
+	  flattenPlusM d (Number.mul a coef) b
+      | Uminus x -> 
+	  flattenPlusM d (Number.negate coef) x
+      |	Plus l -> 
+	  snumM coef >>= fun coef' ->
+	    mapM (fun x -> stimesM (coef', x)) l 
+      |	x -> snumM coef >>= fun coef' ->
+	  stimesM (coef', x) >>= fun x' -> returnM [x']
+    in let l' = List.flatten (List.map (findTerms 1) l)
+    in let d = duplicates l'
+    in if (List.length d) > 0 then
+      mapM (flattenPlusM d Number.one) l >>= fun a ->
+	collectExprM (List.flatten a) >>=
+	mangleSumM
+    else
+      returnM l
 
   and splusM l = mangleSumM l >>=  fun l' ->
   (* no terms are negative.  Don't do anything *)
@@ -539,7 +670,7 @@ end = struct
   else if List.for_all negative l' then
     mapM suminusM l' >>= splusM >>= suminusM
   (* some terms are positive and some are negative.  We are in trouble.
-     Ask the oracle *)
+     Ask the Oracle *)
   else if Oracle.should_flip_sign (Plus l') then
     mapM suminusM l' >>= splusM >>= suminusM
   else
@@ -599,16 +730,22 @@ end
 (* simplify the dag *)
 let algsimp v = 
   let _ = info "  first simplification pass..." in
+  let _ = info (Stats.complexity v) in
   let v = AlgSimp.algsimp v in
   let _ = info "  second simplification pass..." in
+  let _ = info (Stats.complexity v) in
   let v = Reverse.reverse v in
   let _ = info "  third simplification pass..." in
+  let _ = info (Stats.complexity v) in
   let v = AlgSimp.algsimp v in
   let _ = info "  fourth simplification pass..." in
+  let _ = info (Stats.complexity v) in
   let v = Reverse.reverse v in
   let _ = info "  fifth simplification pass..." in
+  let _ = info (Stats.complexity v) in
   let v = AlgSimp.algsimp v in
   let _ = info "  simplification done..." in
+  let _ = info (Stats.complexity v) in
   v
 
 let make nodes = Dag nodes
@@ -799,47 +936,4 @@ let simplify_to_alist dag =
   in List.map wrap_assign (to_assignments d1)
 
 
-module LittleSimplifier = struct 
-  (* 
-   * The LittleSimplifier module implements a subset of the simplifications
-   * of the AlgSimp module.  These simplifications can be executed
-   * quickly here, while they would take a long time using the heavy
-   * machinery of AlgSimp.  
-   * 
-   * For example, 0 * x is simplified to 0 tout court by the LittleSimplifier.
-   * On the other hand, AlgSimp would first simplify x, generating lots
-   * of common subexpressions, storing them in a table etc, just to
-   * discard all the work later.  Similarly, the LittleSimplifier
-   * reduces the constant FFT in Rader's algorithm to a constant sequence.
-   *)
-  let rec makeNum = function
-    | n -> Num n
-
-  and makeUminus = function
-    | Uminus a -> a 
-    | Num a -> makeNum (Number.negate a)
-    | a -> Uminus a
-
-  and makeTimes = function
-    | (Num a, Num b) -> makeNum (Number.mul a b)
-    | (Num a, Times (Num b, c)) -> makeTimes (makeNum (Number.mul a b), c)
-    | (Num a, b) when Number.is_zero a -> makeNum (Number.zero)
-    | (Num a, b) when Number.is_one a -> b
-    | (Num a, b) when Number.is_mone a -> makeUminus b
-    | (a, (Num b as b')) -> makeTimes (b', a)
-    | (a, b) -> Times (a, b)
-
-  and makePlus l = 
-    let rec reduceSum x = match x with
-	[] -> []
-      | [Num a] -> if Number.is_zero a then [] else x
-      | (Num a) :: (Num b) :: c -> 
-	  reduceSum ((makeNum (Number.add a b)) :: c)
-      | ((Num _) as a') :: b :: c -> b :: reduceSum (a' :: c)
-      | a :: s -> a :: reduceSum s
-    in match reduceSum l with
-      [] -> makeNum (Number.zero)
-    | [a] -> a 
-    | a -> Plus a
-end
 
