@@ -1,5 +1,5 @@
 (*
- * Copyright (c) 1997,1998 Massachusetts Institute of Technology
+ * Copyright (c) 1997-1999 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *
  *)
 
-(* $Id: genfft.ml,v 1.66 1998/09/25 16:29:20 fftw Exp $ *)
+(* $Id: genfft.ml,v 1.75 1999/05/26 15:44:15 fftw Exp $ *)
 
 (* This file contains the entry point for the genfft program: it
    parses the command-line parameters and calls the rest of the
@@ -46,6 +46,8 @@ let ostride = "ostride"
 let twiddle_order = "twiddle_order"
 
 type codelet_type = TWIDDLE | NO_TWIDDLE | REAL2HC | HC2HC | HC2REAL
+                    | REALEVEN | REALODD | REALEVEN2 | REALODD2
+                    | REALEVEN_TWIDDLE | REALODD_TWIDDLE
 
 let rec list_to_c = function
     [] -> ""
@@ -60,6 +62,12 @@ let codelet_description n dir ty f =
   | REAL2HC -> "FFTW_REAL2HC", 2
   | HC2HC -> "FFTW_HC2HC", 3
   | HC2REAL -> "FFTW_HC2REAL", 4
+  | REALEVEN -> "FFTW_REALEVEN", 5
+  | REALODD -> "FFTW_REALODD", 6
+  | REALEVEN2 -> "FFTW_REALEVEN2", 7
+  | REALODD2 -> "FFTW_REALODD2", 8
+  | REALEVEN_TWIDDLE -> "FFTW_REALEVEN_TWIDDLE", 9
+  | REALODD_TWIDDLE -> "FFTW_REALODD_TWIDDLE", 10
   and (cdir, idir) = match dir with
     Fft.FORWARD -> "FFTW_FORWARD", 0
   | Fft.BACKWARD -> "FFTW_BACKWARD", 1
@@ -79,9 +87,23 @@ let codelet_description n dir ty f =
 	twiddle_order,
 	num_twiddle n
     | HC2REAL -> "", "(const int *) 0", 0
+    | REALEVEN -> "", "(const int *) 0", 0
+    | REALODD -> "", "(const int *) 0", 0
+    | REALEVEN2 -> "", "(const int *) 0", 0
+    | REALODD2 -> "", "(const int *) 0", 0
+    | REALEVEN_TWIDDLE -> 
+        ("static const int " ^ twiddle_order ^ "[] = {" ^
+                  (list_to_c (tw_o n)) ^ "};\n"),
+        twiddle_order,
+        num_twiddle n
+    | REALODD_TWIDDLE -> 
+        ("static const int " ^ twiddle_order ^ "[] = {" ^
+                  (list_to_c (tw_o n)) ^ "};\n"),
+        twiddle_order,
+        num_twiddle n
 
   (* this should be replaced by CRC/MD5 of the codelet *)
-  and signature = 8 * (2 * n + idir) + itype 
+  and signature = 11 * (2 * n + idir) + itype 
 
   in "\n\n" ^ declare_order ^
   "fftw_codelet_desc " ^ name ^ "_desc = {\n" ^
@@ -130,7 +152,7 @@ let athena_no_twiddle_gen n dir =
 	     [Decl ("fftw_complex *", iarray)],
 	     Asch asch)
   and prologue = 
-    "#include <athfft.h>\n\n" ^
+    "#include \"athfft.h\"\n\n" ^
     (if dir == Fft.FORWARD then
       "int ath_permutation_" ^ ns ^ "(int i) { return i; }\n\n\n"
     else "")
@@ -198,6 +220,84 @@ let hc2real_gen n =
   in let desc = codelet_description n dir HC2REAL tree
   in ((make_c_unparser unparse_var) tree) ^ desc
 
+let realeven_gen n =
+  let _ = info "generating..." in
+  let dir = Fft.FORWARD in
+  let asch = optimize (Fft.no_twiddle_gen_expr n Symmetry.realeven_sym dir)
+  and ns = string_of_int n
+  and (name, sign) = "fftw_realeven_", (-1)
+  and unparse_var = 
+    Variable.make_realeven_unparser (iarray, Some istride) 
+                                    (oarray, Some ostride) 
+  in let tree = 
+	Fcn ("void", name ^ ns,
+	     [Decl ("const fftw_real *", iarray);
+	      Decl ("fftw_real *", oarray);
+	      Decl ("int", istride);
+	      Decl ("int", ostride)],
+	     Asch asch)
+  in let desc = codelet_description n dir REALEVEN tree
+  in ((make_c_unparser unparse_var) tree) ^ desc
+
+let realodd_gen n =
+  let _ = info "generating..." in
+  let dir = Fft.FORWARD in
+  let asch = optimize (Fft.no_twiddle_gen_expr n Symmetry.realodd_sym dir)
+  and ns = string_of_int n
+  and (name, sign) = "fftw_realodd_", (-1)
+  and unparse_var = 
+    Variable.make_realodd_unparser (iarray, Some istride) 
+                                   (oarray, Some ostride) 
+  in let tree = 
+	Fcn ("void", name ^ ns,
+	     [Decl ("const fftw_real *", iarray);
+	      Decl ("fftw_real *", oarray);
+	      Decl ("int", istride);
+	      Decl ("int", ostride)],
+	     Asch asch)
+  in let desc = codelet_description n dir REALODD tree
+  in ((make_c_unparser unparse_var) tree) ^ desc
+
+let realeven2_gen n =
+  let _ = info "generating..." in
+  let dir = Fft.FORWARD in
+  let asch = optimize (Fft.no_twiddle_gen_expr (2 * n)
+                         Symmetry.realeven2_input_sym dir) 
+  and ns = string_of_int n
+  and (name, sign) = "fftw_realeven2_", (-1)
+  and unparse_var = 
+    Variable.make_realeven_unparser (iarray, Some istride) 
+                                    (oarray, Some ostride) 
+  in let tree = 
+	Fcn ("void", name ^ ns,
+	     [Decl ("const fftw_real *", iarray);
+	      Decl ("fftw_real *", oarray);
+	      Decl ("int", istride);
+	      Decl ("int", ostride)],
+	     Asch asch)
+  in let desc = codelet_description n dir REALEVEN2 tree
+  in ((make_c_unparser unparse_var) tree) ^ desc
+
+let realodd2_gen n =
+  let _ = info "generating..." in
+  let dir = Fft.FORWARD in
+  let asch = optimize (Fft.no_twiddle_gen_expr (2 * n)
+			 Symmetry.realodd2_input_sym dir)
+  and ns = string_of_int n
+  and (name, sign) = "fftw_realodd2_", (-1)
+  and unparse_var = 
+    Variable.make_realodd2_unparser (iarray, Some istride) 
+                                    (oarray, Some ostride) 
+  in let tree = 
+	Fcn ("void", name ^ ns,
+	     [Decl ("const fftw_real *", iarray);
+	      Decl ("fftw_real *", oarray);
+	      Decl ("int", istride);
+	      Decl ("int", ostride)],
+	     Asch asch)
+  in let desc = codelet_description n dir REALODD2 tree
+  in ((make_c_unparser unparse_var) tree) ^ desc
+
 let ioarray = "inout"
 let iostride = "iostride"
 let twarray = "W"
@@ -293,7 +393,7 @@ let athena_twiddle_gen n dir =
 	   Decl ("int", iostride)],
          body)
   and prologue = 
-    "#include <athfft.h>\n\n"
+    "#include \"athfft.h\"\n\n"
   in prologue ^ ((make_c_unparser unparse_var) tree)
 
 let twiddle_gen n dir =
@@ -314,7 +414,7 @@ let hc2hc_gen n dir =
 	optimize (Fft.no_twiddle_gen_expr n Symmetry.real_sym dir),
 	optimize (Fft.twiddle_dit_gen_expr n Symmetry.no_sym 
 		    Symmetry.middle_hc2hc_forward_sym dir),
-	optimize (Fft.twiddle_dit_gen_expr n Symmetry.real_sym 
+	optimize (Fft.no_twiddle_gen_expr (2 * n) 
 		    Symmetry.final_hc2hc_forward_sym dir),
 	"fftw_hc2hc_forward_", 
 	(-1),
@@ -323,8 +423,8 @@ let hc2hc_gen n dir =
 	optimize (Fft.no_twiddle_gen_expr n Symmetry.hermitian_sym dir),
 	optimize (Fft.twiddle_dif_gen_expr n Symmetry.middle_hc2hc_backward_sym
 		    Symmetry.no_sym dir),
-	optimize (Fft.twiddle_dif_gen_expr n Symmetry.final_hc2hc_backward_sym
-		    Symmetry.hermitian_sym dir),
+	optimize (Fft.no_twiddle_gen_expr (2 * n) 
+		    Symmetry.final_hc2hc_backward_sym dir),
 	"fftw_hc2hc_backward_", 
 	1,
 	Variable.make_hc2hc_backward_unparser)
@@ -396,6 +496,12 @@ type mode =
   | GEN_HC2REAL of int
   | GEN_HC2HC_FORWARD of int
   | GEN_HC2HC_BACKWARD of int
+  | GEN_REAL_EVEN of int
+  | GEN_REAL_ODD of int
+  | GEN_REAL_EVEN2 of int
+  | GEN_REAL_ODD2 of int
+  | GEN_REAL_EVEN_TWIDDLE of int
+  | GEN_REAL_ODD_TWIDDLE of int
 
 let mode = ref GEN_NOTHING
 
@@ -411,6 +517,12 @@ let main () =
   "-hc2hc-forward", Arg.Int(fun i -> mode := GEN_HC2HC_FORWARD i), "<n> : Generate a forward halfcomplex to halfcomplex codelet of size <n>";
   "-hc2hc-backward", Arg.Int(fun i -> mode := GEN_HC2HC_BACKWARD i), "<n> : Generate a backward halfcomplex to halfcomplex codelet of size <n>";
   "-hc2real", Arg.Int(fun i -> mode := GEN_HC2REAL i), "<n> : Generate a halfcomplex to real codelet of size <n>";
+  "-realeven", Arg.Int(fun i -> mode := GEN_REAL_EVEN i), "<n> : Generate a real even codelet of size <n>";
+  "-realodd", Arg.Int(fun i -> mode := GEN_REAL_ODD i), "<n> : Generate a real odd codelet of size <n>";
+  "-realeven2", Arg.Int(fun i -> mode := GEN_REAL_EVEN2 i), "<n> : Generate a real even-2 codelet of size <n>";
+  "-realodd2", Arg.Int(fun i -> mode := GEN_REAL_ODD2 i), "<n> : Generate a real odd-2 codelet of size <n>";
+  "-realeventwiddle", Arg.Int(fun i -> mode := GEN_REAL_EVEN_TWIDDLE i), "<n> : Generate a real even twiddle codelet of size <n>";
+  "-realoddtwiddle", Arg.Int(fun i -> mode := GEN_REAL_ODD_TWIDDLE i), "<n> : Generate a real odd twiddle codelet of size <n>";
 
   "-verbose", 
   Arg.Unit(fun () -> Magic.verbose := true),
@@ -446,6 +558,13 @@ let main () =
   Arg.Unit(fun () -> Magic.loopo := false),
   undocumented;
 
+  "-magic-times-3-3", 
+  Arg.Unit(fun () -> Magic.times_3_3 := true),
+  undocumented;
+  "-magic-times-4-2", 
+  Arg.Unit(fun () -> Magic.times_3_3 := false),
+  undocumented;
+
   "-magic-inline-single", 
   Arg.Unit(fun () -> Magic.inline_single := true),
   undocumented;
@@ -529,6 +648,10 @@ let main () =
     | GEN_HC2HC_FORWARD i -> (hc2hc_gen i Fft.FORWARD)
     | GEN_HC2REAL i -> (hc2real_gen i)
     | GEN_HC2HC_BACKWARD i -> (hc2hc_gen i Fft.BACKWARD)
+    | GEN_REAL_EVEN i -> (realeven_gen i)
+    | GEN_REAL_ODD i -> (realodd_gen i)
+    | GEN_REAL_EVEN2 i -> (realeven2_gen i)
+    | GEN_REAL_ODD2 i -> (realodd2_gen i)
     | _ -> failwith "one of -notwiddle, -notwiddleinv, -twiddle, -twiddleinv, -real2hc, -hc2real, -hc2hc-forward, or -hc2hc-forward must be specified"
 
   in begin
