@@ -1,6 +1,6 @@
 
 /*
- * Copyright (c) 1997,1998 Massachusetts Institute of Technology
+ * Copyright (c) 1997-1999 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *
  */
 
-/* $Id: rfftwnd.c,v 1.30 1998/09/27 00:10:29 stevenj Exp $ */
+/* $Id: rfftwnd.c,v 1.33 1999/11/02 23:52:02 stevenj Exp $ */
 
 #include <fftw-int.h>
 #include <rfftw.h>
@@ -314,8 +314,8 @@ void rfftwnd_real2c_aux_howmany(fftwnd_plan p, int cur_dim,
 		    rfftw_real2c_overlap_aux(p->plans[p->rank - 1], howmany,
 					in + (k * n_after * istride) * 2,
 					     istride, idist,
-					   out + (k * n_after * istride),
-					     istride, idist,
+					   out + (k * n_after * ostride),
+					     ostride, odist,
 					     (fftw_real *) work);
 	  else {
 	       int nlast = p->plans[p->rank - 1]->n;
@@ -373,8 +373,8 @@ void rfftwnd_c2real_aux_howmany(fftwnd_plan p, int cur_dim,
 		    rfftw_c2real_overlap_aux(p->plans[p->rank - 1], howmany,
 					     in + (k * n_after * istride),
 					     istride, idist,
-				       out + (k * n_after * istride) * 2,
-					     istride, idist,
+				       out + (k * n_after * ostride) * 2,
+					     ostride, odist,
 					     (fftw_real *) work);
 	  else {
 	       int nlast = p->plans[p->rank - 1]->n;
@@ -425,7 +425,7 @@ void rfftwnd_real_to_complex(fftwnd_plan p, int howmany,
 
      if (p->is_in_place) {
 	  ostride = istride;
-	  odist = (idist == 1) ? 1 : (idist / 2);	/* ugh */
+	  odist = (idist == 1 && idist < istride) ? 1 : (idist / 2);  /* ugh */
 	  out = (fftw_complex *) in;
 	  if (howmany > 1 && istride > idist && rank > 0) {
 	       int new_nwork;
@@ -502,7 +502,7 @@ void rfftwnd_complex_to_real(fftwnd_plan p, int howmany,
      if (p->is_in_place) {
 	  ostride = istride;
 	  odist = idist;
-	  odist = (idist == 1) ? 1 : (idist * 2);	/* ugh */
+	  odist = (idist == 1 && idist < istride) ? 1 : (idist * 2);  /* ugh */
 	  out = (fftw_real *) in;
 	  if (howmany > 1 && istride > idist && rank > 0) {
 	       int new_nwork = p->n[rank - 1] * howmany;
