@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997,1998 Massachusetts Institute of Technology
+ * Copyright (c) 1997-1999 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,8 +24,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <fftw-int.h>
-#include <fftw_threads.h>
+#include <fftw_threads-int.h>
 
 static void executor_simple_threads(int n, const fftw_complex *in,
 				    fftw_complex *out,
@@ -35,57 +34,49 @@ static void executor_simple_threads(int n, const fftw_complex *in,
 				    int nthreads);
 
 typedef struct {
-  int min, max;
-  int m,r;
-  const fftw_complex *in;
-  fftw_complex *out;
-  fftw_plan_node *p;
-  int istride, ostride;
-  int nthreads;
-} executor_simple_data;
-
-static void *executor_simple_thread(executor_simple_data *d)
-{
-     int min, max, m, r;
+     int m,r;
      const fftw_complex *in;
      fftw_complex *out;
      fftw_plan_node *p;
      int istride, ostride;
      int nthreads;
+} executor_simple_data;
+
+static void *executor_simple_thread(fftw_loop_data *loop_data)
+{
+     int min = loop_data->min, max = loop_data->max;
+     executor_simple_data *d = (executor_simple_data *) loop_data->data;
+     int m = d->m, r = d->r;
+     const fftw_complex *in = d->in;
+     fftw_complex *out = d->out;
+     fftw_plan_node *p = d->p;
+     int istride = d->istride, ostride = d->ostride;
+     int nthreads = d->nthreads;
      
-     min = d->min; max = d->max;
-     m = d->m; r = d->r;
-     in = d->in; out = d->out;
-     p = d->p;
-     istride = d->istride;
-     ostride = d->ostride;
-     nthreads = d->nthreads;
-     
-     for (; min < max; ++min) {
+     for (; min < max; ++min)
 	  executor_simple_threads(m, in + min * istride,
 				  out + min * (m * ostride),
 				  p,
 				  istride * r, ostride,
 				  nthreads);
-     }
 
      return 0;
 }
 
 typedef struct {
-     int min, max;
      fftw_twiddle_codelet *codelet;
-     int m,r, ostride;
+     int m, ntwiddle, ostride;
      fftw_complex *out, *W;
 } twiddle_thread_data;
 
-static void *twiddle_thread(twiddle_thread_data *d)
+static void *twiddle_thread(fftw_loop_data *loop_data)
 {
-     HACK_ALIGN_STACK_EVEN();
-     (d->codelet)(d->out + d->ostride * d->min,
-		  d->W + (d->r - 1) * d->min,
+     twiddle_thread_data *d = (twiddle_thread_data *) loop_data->data;     
+     HACK_ALIGN_STACK_EVEN;
+     (d->codelet)(d->out + d->ostride * loop_data->min,
+		  d->W + d->ntwiddle * loop_data->min,
 		  d->m * d->ostride,
-		  d->max - d->min,
+		  loop_data->max - loop_data->min,
 		  d->ostride);
      return 0;
 }
@@ -99,7 +90,7 @@ static void executor_simple_threads(int n, const fftw_complex *in,
 {
      switch (p->type) {
 	 case FFTW_NOTW:
-              HACK_ALIGN_STACK_ODD();
+              HACK_ALIGN_STACK_ODD;
 	      (p->nodeu.notw.codelet) (in, out, istride, ostride);
 	      break;
 
@@ -117,61 +108,41 @@ static void executor_simple_threads(int n, const fftw_complex *in,
 			     fftw_executor_simple(m, in + i * istride,
 						  out + i * (m * ostride),
 						  p->nodeu.twiddle.recurse,
-						  istride * r, ostride);
+						  istride * r, ostride,
+						  FFTW_NORMAL_RECURSE);
 			}
 			codelet = p->nodeu.twiddle.codelet;
 			W = p->nodeu.twiddle.tw->twarray;
-			HACK_ALIGN_STACK_EVEN();
+			HACK_ALIGN_STACK_EVEN;
 			codelet(out, W, m * ostride, m, ostride);
 		   }
 		   else {
 			{
-			     executor_simple_data *d;
-			     int subthread_nthreads, new_nthreads;
+			     executor_simple_data d;
 			     
-			     if (nthreads > r) {
-				  subthread_nthreads = nthreads / r;
-				  new_nthreads = r;
-			     }
-			     else {
-				  subthread_nthreads = 1;
-				  new_nthreads = nthreads;
-			     }
-
-			     d = alloca(new_nthreads * 
-					sizeof(executor_simple_data));
+			     d.m = m; d.r = r;
+			     d.in = in; d.out = out;
+			     d.p = p->nodeu.twiddle.recurse;
+			     d.istride = istride;
+			     d.ostride = ostride;
+			     d.nthreads = nthreads / r;
 			     
-			     d->m = m; d->r = r;
-			     d->in = in; d->out = out;
-			     d->p = p->nodeu.twiddle.recurse;
-			     d->istride = istride;
-			     d->ostride = ostride;
-			     d->nthreads = subthread_nthreads;
-			     for (i = 1; i < new_nthreads; ++i)
-				  d[i] = *d;
-			     
-			     fftw_thread_spawn_loop(r,new_nthreads,
-						    executor_simple_thread,d);
+			     fftw_thread_spawn_loop(r, nthreads,
+						    executor_simple_thread,&d);
 			}
 			{
-			     twiddle_thread_data *d;
+			     twiddle_thread_data d;
 
-			     if (nthreads > m)
-				  nthreads = m;
+			     d.codelet = p->nodeu.twiddle.codelet;
+			     d.m = m;
+			     d.ntwiddle =
+				  p->nodeu.twiddle.codelet_desc->ntwiddle;
+			     d.ostride = ostride;
+			     d.out = out;
+			     d.W = p->nodeu.twiddle.tw->twarray;
 
-			     d = alloca(nthreads * 
-					sizeof(twiddle_thread_data));
-			     d->codelet = p->nodeu.twiddle.codelet;
-			     d->m = m;
-			     d->r = r;
-			     d->ostride = ostride;
-			     d->out = out;
-			     d->W = p->nodeu.twiddle.tw->twarray;
-			     for (i = 1; i < nthreads; ++i)
-				  d[i] = *d;
-
-			     fftw_thread_spawn_loop(m,nthreads,
-						    twiddle_thread,d);
+			     fftw_thread_spawn_loop(m, nthreads,
+						    twiddle_thread, &d);
 			}
 		   }
 
@@ -192,7 +163,8 @@ static void executor_simple_threads(int n, const fftw_complex *in,
 			     fftw_executor_simple(m, in + i * istride,
 						  out + i * (m * ostride),
 						  p->nodeu.rader.recurse,
-						  istride * r, ostride);
+						  istride * r, ostride,
+						  FFTW_NORMAL_RECURSE);
 			}
 			codelet = p->nodeu.rader.codelet;
 			W = p->nodeu.rader.tw->twarray;
@@ -201,32 +173,17 @@ static void executor_simple_threads(int n, const fftw_complex *in,
 		   }
 		   else {
 			{
-			     executor_simple_data *d;
-			     int subthread_nthreads, new_nthreads;
-			     
-			     if (nthreads > r) {
-				  subthread_nthreads = nthreads / r;
-				  new_nthreads = r;
-			     }
-			     else {
-				  subthread_nthreads = 1;
-				  new_nthreads = nthreads;
-			     }
+			     executor_simple_data d;
 
-			     d = alloca(new_nthreads * 
-					sizeof(executor_simple_data));
+			     d.m = m; d.r = r;
+			     d.in = in; d.out = out;
+			     d.p = p->nodeu.rader.recurse;
+			     d.istride = istride;
+			     d.ostride = ostride;
+			     d.nthreads = nthreads / r;
 			     
-			     d->m = m; d->r = r;
-			     d->in = in; d->out = out;
-			     d->p = p->nodeu.rader.recurse;
-			     d->istride = istride;
-			     d->ostride = ostride;
-			     d->nthreads = subthread_nthreads;
-			     for (i = 1; i < new_nthreads; ++i)
-				  d[i] = *d;
-			     
-			     fftw_thread_spawn_loop(r,new_nthreads,
-						    executor_simple_thread,d);
+			     fftw_thread_spawn_loop(r, nthreads,
+						    executor_simple_thread,&d);
 			}
 			{
 			     fftw_rader_codelet *codelet;
@@ -255,29 +212,21 @@ static void executor_simple_threads(int n, const fftw_complex *in,
 			     fftw_executor_simple(m, in + i * istride,
 						  out + i * (m * ostride),
 						  p->nodeu.generic.recurse,
-						  istride * r, ostride);
+						  istride * r, ostride,
+						  FFTW_NORMAL_RECURSE);
 			}
 		   else {
-			executor_simple_data *d;
+			executor_simple_data d;
 
-			d = alloca(nthreads * sizeof(executor_simple_data));
+			d.m = m; d.r = r;
+			d.in = in; d.out = out;
+			d.p = p->nodeu.generic.recurse;
+			d.istride = istride;
+			d.ostride = ostride;
+			d.nthreads = nthreads / r;
 
-			d->m = m; d->r = r;
-			d->in = in; d->out = out;
-			d->p = p->nodeu.generic.recurse;
-			d->istride = istride;
-			d->ostride = ostride;
-			if (nthreads > r) {
-			     d->nthreads = nthreads / r;
-			     nthreads = r;
-			}
-			else
-			     d->nthreads = 1;
-			for (i = 1; i < nthreads; ++i)
-			     d[i] = *d;
-
-			fftw_thread_spawn_loop(r,nthreads,
-					       executor_simple_thread,d);
+			fftw_thread_spawn_loop(r, nthreads,
+					       executor_simple_thread, &d);
 		   }
 
 		   codelet = p->nodeu.generic.codelet;
@@ -301,28 +250,31 @@ static void executor_simple_inplace_threads(int n, fftw_complex *in,
 {
      switch (p->type) {
 	 case FFTW_NOTW:
-              HACK_ALIGN_STACK_ODD();
+              HACK_ALIGN_STACK_ODD;
 	      (p->nodeu.notw.codelet) (in, in, istride, istride);
 	      break;
 
 	 default:
 	      {
 		   fftw_complex *tmp;
-
-		   tmp = (fftw_complex *)
-			fftw_malloc(n * sizeof(fftw_complex));
+		   
+		   if (out)
+			tmp = out;
+		   else
+			tmp = (fftw_complex *)
+			     fftw_malloc(n * sizeof(fftw_complex));
 		   
 		   executor_simple_threads(n, in, tmp, p, istride, 1, 
 					   nthreads);
 		   fftw_strided_copy(n, tmp, istride, in);
 
-		   fftw_free(tmp);
+		   if (!out)
+			fftw_free(tmp);
 	      }
      }
 }
 
 typedef struct {
-     int min, max;
      union {
 	  fftw_notw_codelet *codelet;
 	  struct {
@@ -331,28 +283,21 @@ typedef struct {
 	  } plan;
      } u;
      const fftw_complex *in;
-     fftw_complex*out;
+     fftw_complex *out;
      int idist, odist, istride, ostride;
 } executor_many_data;
 
-static void *executor_many_codelet_thread(executor_many_data *d)
+static void *executor_many_codelet_thread(fftw_loop_data *loop_data)
 {
-     int min, max;
-     fftw_notw_codelet *codelet;
-     const fftw_complex *in;
-     fftw_complex *out;
-     int idist, odist, istride, ostride;
+     int min = loop_data->min, max = loop_data->max;
+     executor_many_data *d = (executor_many_data *) loop_data->data;
+     fftw_notw_codelet *codelet = d->u.codelet;
+     const fftw_complex *in = d->in;
+     fftw_complex *out = d->out;
+     int idist = d->idist, odist = d->odist;
+     int istride = d->istride, ostride = d->ostride;
      
-     min = d->min; max = d->max;
-     codelet = d->u.codelet;
-     in = d->in;
-     out = d->out;
-     idist = d->idist;
-     odist = d->odist;
-     istride = d->istride;
-     ostride = d->ostride;
-
-     HACK_ALIGN_STACK_ODD();
+     HACK_ALIGN_STACK_ODD;
      for (; min < max; ++min)
 	  codelet(in + min * idist,
 		  out + min * odist,
@@ -361,29 +306,22 @@ static void *executor_many_codelet_thread(executor_many_data *d)
      return 0;
 }
 
-static void *executor_many_simple_thread(executor_many_data *d)
+static void *executor_many_simple_thread(fftw_loop_data *loop_data)
 {
-     int min, max;
-     int n;
-     fftw_plan_node *p;
-     const fftw_complex *in;
-     fftw_complex *out;
-     int idist, odist, istride, ostride;
-     
-     min = d->min; max = d->max;
-     n = d->u.plan.n;
-     p = d->u.plan.p;
-     in = d->in;
-     out = d->out;
-     idist = d->idist;
-     odist = d->odist;
-     istride = d->istride;
-     ostride = d->ostride;
+     int min = loop_data->min, max = loop_data->max;
+     executor_many_data *d = (executor_many_data *) loop_data->data;
+     int n = d->u.plan.n;
+     fftw_plan_node *p = d->u.plan.p;
+     const fftw_complex *in = d->in;
+     fftw_complex *out = d->out;
+     int idist = d->idist, odist = d->odist;
+     int istride = d->istride, ostride = d->ostride;
 
      for (; min < max; ++min)
 	  fftw_executor_simple(n, in + min * idist,
 			       out + min * odist,
-			       p, istride, ostride);
+			       p, istride, ostride,
+			       FFTW_NORMAL_RECURSE);
 
      return 0;
 }
@@ -396,9 +334,6 @@ static void executor_many_threads(int n, const fftw_complex *in,
 				  int howmany, int idist, int odist,
 				  int nthreads)
 {
-     if (nthreads > howmany)
-	  nthreads = howmany;     
-
      switch (p->type) {
 	 case FFTW_NOTW:
 	      {
@@ -406,28 +341,24 @@ static void executor_many_threads(int n, const fftw_complex *in,
 		   
 		   if (nthreads <= 1) {
 			fftw_notw_codelet *codelet = p->nodeu.notw.codelet;
-			HACK_ALIGN_STACK_ODD();
+			HACK_ALIGN_STACK_ODD;
 			for (s = 0; s < howmany; ++s)
 			     codelet(in + s * idist,
 				     out + s * odist,
 				     istride, ostride);
 		   }
 		   else {
-			executor_many_data *d;
-			
-			d = alloca(nthreads * sizeof(executor_many_data));
-			
-			d->in = in;
-			d->out = out;
-			d->u.codelet = p->nodeu.notw.codelet;
-			d->istride = istride;
-			d->ostride = ostride;
-			d->idist = idist;
-			d->odist = odist;
-			for (s = 1; s < nthreads; ++s)
-			     d[s] = *d;
-			fftw_thread_spawn_loop(howmany,nthreads,
-				   executor_many_codelet_thread,d);
+			executor_many_data d;
+
+			d.in = in;
+			d.out = out;
+			d.u.codelet = p->nodeu.notw.codelet;
+			d.istride = istride;
+			d.ostride = ostride;
+			d.idist = idist;
+			d.odist = odist;
+			fftw_thread_spawn_loop(howmany, nthreads,
+				   executor_many_codelet_thread, &d);
 		   }
 
 		   break;
@@ -441,31 +372,27 @@ static void executor_many_threads(int n, const fftw_complex *in,
 			for (s = 0; s < howmany; ++s) {
 			     fftw_executor_simple(n, in + s * idist,
 						  out + s * odist,
-						  p, istride, ostride);
+						  p, istride, ostride,
+						  FFTW_NORMAL_RECURSE);
 			}
 		   else {
-			executor_many_data *d;
-			
-			d = alloca(nthreads * sizeof(executor_many_data));
-			
-			d->in = in; d->out = out;
-			d->u.plan.n = n;
-			d->u.plan.p = p;
-			d->istride = istride;
-			d->ostride = ostride;
-			d->idist = idist;
-			d->odist = odist;
-			for (s = 1; s < nthreads; ++s)
-			     d[s] = *d;
-			fftw_thread_spawn_loop(howmany,nthreads,
-				   executor_many_simple_thread,d);
+			executor_many_data d;
+
+			d.in = in; d.out = out;
+			d.u.plan.n = n;
+			d.u.plan.p = p;
+			d.istride = istride;
+			d.ostride = ostride;
+			d.idist = idist;
+			d.odist = odist;
+			fftw_thread_spawn_loop(howmany, nthreads,
+				   executor_many_simple_thread, &d);
 		   }
 	      }
      }
 }
 
 typedef struct {
-     int min, max;
      union {
 	  fftw_notw_codelet *codelet;
 	  struct {
@@ -478,20 +405,16 @@ typedef struct {
      int idist, istride;
 } executor_many_inplace_data;
 
-static void *executor_many_inplace_codelet_thread(executor_many_inplace_data *d)
+static void *executor_many_inplace_codelet_thread(fftw_loop_data *loop_data)
 {
-     int min, max;
-     fftw_notw_codelet *codelet;
-     fftw_complex *in;
-     int idist, istride;
+     int min = loop_data->min, max = loop_data->max;
+     executor_many_inplace_data
+	  *d = (executor_many_inplace_data *) loop_data->data;
+     fftw_notw_codelet *codelet = d->u.codelet;
+     fftw_complex *in = d->in;
+     int idist = d->idist, istride = d->istride;
      
-     min = d->min; max = d->max;
-     codelet = d->u.codelet;
-     in = d->in;
-     idist = d->idist;
-     istride = d->istride;
-
-     HACK_ALIGN_STACK_ODD();
+     HACK_ALIGN_STACK_ODD;
      for (; min < max; ++min)
 	  codelet(in + min * idist,
 		  in + min * idist,
@@ -500,42 +423,34 @@ static void *executor_many_inplace_codelet_thread(executor_many_inplace_data *d)
      return 0;
 }
 
-static void *executor_many_inplace_simple_thread(executor_many_inplace_data *d)
+static void *executor_many_inplace_simple_thread(fftw_loop_data *loop_data)
 {
-     int min, max;
-     int n;
-     fftw_plan_node *p;
-     fftw_complex *in, *tmp;
-     int idist, istride;
-     
-     min = d->min; max = d->max;
-     n = d->u.plan.n;
-     p = d->u.plan.p;
-     tmp = d->u.plan.tmp;
-     in = d->in;
-     idist = d->idist;
-     istride = d->istride;
+     int min = loop_data->min, max = loop_data->max;
+     executor_many_inplace_data
+          *d = (executor_many_inplace_data *) loop_data->data;
+     int n = d->u.plan.n;
+     fftw_plan_node *p = d->u.plan.p;
+     fftw_complex *tmp = d->u.plan.tmp + n * loop_data->thread_num;
+     fftw_complex *in = d->in;
+     int idist = d->idist, istride = d->istride;
 
      for (; min < max; ++min) {
 	  fftw_executor_simple(n, in + min * idist,
 			       tmp,
-			       p, istride, 1);
+			       p, istride, 1, FFTW_NORMAL_RECURSE);
 	  fftw_strided_copy(n, tmp, istride, in + min * idist);
      }
 
      return 0;
 }
 
-static void executor_many_inplace_threads(int n, fftw_complex *in,
-					  fftw_complex *out,
-					  fftw_plan_node *p,
-					  int istride,
-					  int howmany, int idist,
-					  int nthreads)
+void fftw_executor_many_inplace_threads(int n, fftw_complex *in,
+					fftw_complex *work,
+					fftw_plan_node *p,
+					int istride,
+					int howmany, int idist,
+					int nthreads)
 {
-     if (nthreads > howmany)
-	  nthreads = howmany;     
-
      switch (p->type) {
 	 case FFTW_NOTW:
 	      {
@@ -543,26 +458,22 @@ static void executor_many_inplace_threads(int n, fftw_complex *in,
 
 		   if (nthreads <= 1) {
 			fftw_notw_codelet *codelet = p->nodeu.notw.codelet;
-			HACK_ALIGN_STACK_ODD();
+			HACK_ALIGN_STACK_ODD;
 			for (s = 0; s < howmany; ++s)
 			     codelet(in + s * idist,
 				     in + s * idist,
 				     istride, istride);
 		   }
 		   else {
-			executor_many_inplace_data *d;
+			executor_many_inplace_data d;
 			
-			d = alloca(nthreads * 
-				   sizeof(executor_many_inplace_data));
-			
-			d->in = in;
-			d->u.codelet = p->nodeu.notw.codelet;
-			d->istride = istride;
-			d->idist = idist;
-			for (s = 1; s < nthreads; ++s)
-			     d[s] = *d;
-			fftw_thread_spawn_loop(howmany,nthreads,
-				   executor_many_inplace_codelet_thread,d);
+			d.in = in;
+			d.u.codelet = p->nodeu.notw.codelet;
+			d.istride = istride;
+			d.idist = idist;
+
+			fftw_thread_spawn_loop(howmany, nthreads,
+				   executor_many_inplace_codelet_thread, &d);
 		   }
 		   break;
 	      }
@@ -573,42 +484,45 @@ static void executor_many_inplace_threads(int n, fftw_complex *in,
 		   fftw_complex *tmp;
 
 		   if (nthreads <= 1) {
-			tmp = (fftw_complex *)
-			     fftw_malloc(n * sizeof(fftw_complex));
+			if (work)
+			     tmp = work;
+			else
+			     tmp = (fftw_complex *)
+				  fftw_malloc(n * sizeof(fftw_complex));
 
 			for (s = 0; s < howmany; ++s) {
 			     fftw_executor_simple(n,
 						  in + s * idist,
 						  tmp,
-						  p, istride, 1);
+						  p, istride, 1,
+						  FFTW_NORMAL_RECURSE);
 			     fftw_strided_copy(n, tmp, istride, 
 					       in + s * idist);
 			}
 		   }
 		   else {
-			executor_many_inplace_data *d;
+			executor_many_inplace_data d;
 			
-			tmp = (fftw_complex *)
-			     fftw_malloc(nthreads * n * sizeof(fftw_complex));
-
-			d = alloca(nthreads * 
-				   sizeof(executor_many_inplace_data));
+			if (work)
+			     tmp = work;
+			else
+			     tmp = (fftw_complex *)
+				  fftw_malloc((nthreads > howmany ? 
+					       howmany : nthreads) 
+					      * n * sizeof(fftw_complex));
 			
-			d->in = in;
-			d->u.plan.n = n;
-			d->u.plan.p = p;
-			d->u.plan.tmp = tmp;
-			d->istride = istride;
-			d->idist = idist;
-			for (s = 1; s < nthreads; ++s) {
-			     d[s] = *d;
-			     d[s].u.plan.tmp = tmp + n*s;
-			}
-			fftw_thread_spawn_loop(howmany,nthreads,
-				   executor_many_inplace_simple_thread,d);
+			d.in = in;
+			d.u.plan.n = n;
+			d.u.plan.p = p;
+			d.u.plan.tmp = tmp;
+			d.istride = istride;
+			d.idist = idist;
+			fftw_thread_spawn_loop(howmany, nthreads,
+				   executor_many_inplace_simple_thread, &d);
 		   }
 
-		   fftw_free(tmp);
+		   if (!work)
+			fftw_free(tmp);
 	      }
      }
 }
@@ -625,8 +539,9 @@ void fftw_threads(int nthreads,
 	       executor_simple_inplace_threads(n, in, out, plan->root, 
 					       istride, nthreads);
 	  } else {
-	       executor_many_inplace_threads(n, in, out, plan->root, istride, 
-					     howmany, idist, nthreads);
+	       fftw_executor_many_inplace_threads(n, in, NULL,
+						  plan->root, istride, 
+						  howmany, idist, nthreads);
 	  }
      } else {
 	  if (howmany == 1) {
@@ -638,3 +553,14 @@ void fftw_threads(int nthreads,
 	  }
      }
 }
+
+void fftw_threads_one(int nthreads,
+		      fftw_plan plan, fftw_complex *in, fftw_complex *out)
+{
+     if (plan->flags & FFTW_IN_PLACE)
+	  executor_simple_inplace_threads(plan->n, in, out, plan->root,
+					  1, nthreads);
+     else
+	  executor_simple_threads(plan->n, in, out, plan->root,
+				  1, 1, nthreads);
+}
