@@ -1,6 +1,6 @@
 
 /*
- * Copyright (c) 1997,1998 Massachusetts Institute of Technology
+ * Copyright (c) 1997-1999 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,37 +20,27 @@
 
 #include <stdlib.h>
 
-#include <fftw-int.h>
+#include <fftw_threads-int.h>
 #include <fftw_threads.h>
 
 typedef struct {
-     int min, max;
      fftwnd_plan plan;
      int cur_dim;
      int distance;
      fftw_complex *in, *out;
      int istride, ostride;
-     fftw_complex *tmp;
+     fftw_complex *work;
 } fftwnd_aux_many_data;
 
-static void *fftwnd_aux_many_thread(fftwnd_aux_many_data *d)
+static void *fftwnd_aux_many_thread(fftw_loop_data *loop_data)
 {
-     int min, max;
-     int distance, cur_dim;
-     fftwnd_plan plan;
-     fftw_complex *in, *out;
-     int istride, ostride;
-     fftw_complex *work;
-
-     min = d->min; max = d->max;
-     distance = d->distance;
-     cur_dim = d->cur_dim;
-     plan = d->plan;
-     in = d->in;
-     out = d->out;
-     istride = d->istride;
-     ostride = d->ostride;
-     work = d->tmp;
+     int min = loop_data->min, max = loop_data->max;
+     fftwnd_aux_many_data *d = (fftwnd_aux_many_data *) loop_data->data;
+     int distance = d->distance, cur_dim = d->cur_dim;
+     fftwnd_plan plan = d->plan;
+     fftw_complex *in = d->in, *out = d->out;
+     int istride = d->istride, ostride = d->ostride;
+     fftw_complex *work = d->work + loop_data->thread_num * plan->nwork;
 
      for (; min < max; ++min)
 	  fftwnd_aux(plan,cur_dim,
@@ -66,9 +56,8 @@ static void fftwnd_aux_many_threads(int nthreads, int n, int n_after,
 				    fftw_complex *in, int istride,
 				    fftw_complex *out, int ostride)
 {
-     int i;
      fftw_complex *tmp;
-     fftwnd_aux_many_data *d;
+     fftwnd_aux_many_data d;
 
      if (nthreads > n)
 	  nthreads = n;
@@ -76,21 +65,16 @@ static void fftwnd_aux_many_threads(int nthreads, int n, int n_after,
      tmp = (fftw_complex *) fftw_malloc(nthreads * plan->nwork
 					* sizeof(fftw_complex));
      
-     d = alloca(nthreads * sizeof(fftwnd_aux_many_data));
-     d->plan = plan;
-     d->cur_dim = cur_dim;
-     d->distance = n_after;
-     d->in = in;
-     d->out = out;
-     d->istride = istride;
-     d->ostride = ostride;
-     d->tmp = tmp;
-     for (i = 1; i < nthreads; ++i) {
-	  d[i] = *d;
-	  d[i].tmp = tmp + i * plan->nwork;
-     }
+     d.plan = plan;
+     d.cur_dim = cur_dim;
+     d.distance = n_after;
+     d.in = in;
+     d.out = out;
+     d.istride = istride;
+     d.ostride = ostride;
+     d.work = tmp;
      
-     fftw_thread_spawn_loop(n,nthreads,fftwnd_aux_many_thread,d);
+     fftw_thread_spawn_loop(n, nthreads, fftwnd_aux_many_thread, &d);
 
      fftw_free(tmp);
 }
@@ -163,3 +147,9 @@ void fftwnd_threads(int nthreads, fftwnd_plan p, int howmany,
 	 }
      }
 }
+
+void fftwnd_threads_one(int nthreads, fftwnd_plan p,
+			fftw_complex *in, fftw_complex *out)
+{
+     fftwnd_threads(nthreads, p, 1, in, 1, 0, out, 1, 0);
+}
